@@ -1,4 +1,4 @@
-# (c) 2020 Michał Górny
+# (c) 2020-2021 Michał Górny
 # 2-clause BSD license
 
 """ CLI for nattka. """
@@ -32,7 +32,8 @@ from nattka.package import (find_repository, match_package_list,
                             expand_package_list, ExpandImpossible,
                             format_results, filter_prefix_keywords,
                             PackageKeywordsDict, get_suggested_keywords,
-                            load_profiles, MaskReason)
+                            load_profiles, MaskReason,
+                            can_allarches_for_keywords)
 
 try:
     from nattka.depgraph import (get_ordered_nodes,
@@ -131,11 +132,24 @@ class NattkaCommands(object):
             })
         if getattr(self.args, 'category', []):
             kwargs['category'] = self.args.category
-        if getattr(self.args, 'security', False):
-            kwargs['security'] = True
         if arch:
             kwargs['cc'] = sorted([f'{x}@gentoo.org' for x in arch])
         bugs = bz.find_bugs(**kwargs)
+
+        # hack: Bugzilla seems to suffer from a race condition that can
+        # result in closed bugs being returned when a bug is closed
+        # during the search (as in, actually returned as closed)
+        if not self.args.bug:
+            for bugno, bug in list(bugs.items()):
+                if bug.resolved:
+                    del bugs[bugno]
+
+        # manually filter security bugs due to complex condition
+        if getattr(self.args, 'security', False):
+            for bugno, bug in list(bugs.items()):
+                if not bug.security and 'SECURITY' not in bug.keywords:
+                    del bugs[bugno]
+
         bugnos = list(bugs)
         # if user did not specify explicit list of bugs, start with
         # newest
@@ -175,7 +189,7 @@ class NattkaCommands(object):
 
         repo = self.get_repository()
         git_repo = GitWorkTree(repo.location)
-        if git_repo.path != Path(repo.location):
+        if not git_repo.path.samefile(repo.location):
             log.critical(
                 f'{repo.location} does not seem to be a git repository')
             raise SystemExit(1)
@@ -388,9 +402,14 @@ class NattkaCommands(object):
                 initial_arches = '*'
             else:
                 initial_arches = ' '.join(self.args.arch)
+            if self.args.stabilization:
+                bug_cat = BugCategory.STABLEREQ
+                pkg_attr = 'cpvstr'
+            else:
+                bug_cat = BugCategory.KEYWORDREQ
+                pkg_attr = 'key'
 
-            b = BugInfo(BugCategory.KEYWORDREQ,
-                        f'{packages[0]} {initial_arches}\n')
+            b = BugInfo(bug_cat, f'{packages[0]} {initial_arches}\n')
             plist = dict(match_package_list(repo, b, only_new=True))
             assert len(plist) == 1
             cc_arches = sorted(
@@ -400,9 +419,7 @@ class NattkaCommands(object):
 
             it = 1
             # prepare the initial set
-            b = BugInfo(BugCategory.KEYWORDREQ,
-                        '\n'.join(packages),
-                        cc=cc_arches)
+            b = BugInfo(bug_cat, '\n'.join(packages), cc=cc_arches)
             new_plist = dict(match_package_list(repo, b, only_new=True))
             add_keywords(plist.items(),
                          b.category == BugCategory.STABLEREQ)
@@ -424,8 +441,12 @@ class NattkaCommands(object):
                         # TODO: handle USE-deps meaningfully
                         # TODO: handle <-deps
                         r = atom(d, eapi=eapi).no_usedeps
-                        for m in repo.itermatch(r):
-                            new_packages.add(m.key)
+                        for m in reversed(sorted(repo.itermatch(r))):
+                            if b.category == BugCategory.STABLEREQ:
+                                # skip unkeyworded ebuilds
+                                if not m.keywords:
+                                    continue
+                            new_packages.add(getattr(m, pkg_attr))
                             break
                         else:
                             log.error(f'No match for dependency: {d}')
@@ -436,12 +457,10 @@ class NattkaCommands(object):
                     f'New packages: {" ".join(sorted(new_packages))}')
 
                 # apply on *new* packages
-                b = BugInfo(BugCategory.KEYWORDREQ,
-                            '\n'.join(new_packages),
-                            cc=cc_arches)
+                b = BugInfo(bug_cat, '\n'.join(new_packages), cc=cc_arches)
                 new_plist = dict(match_package_list(repo, b, only_new=True))
                 for p in list(new_packages):
-                    if not any(x.key == p for x in new_plist):
+                    if not any(getattr(x, pkg_attr) == p for x in new_plist):
                         log.info(f'Package {p} seems to be a red herring '
                                  f'(already keyworded everywhere)')
                         new_packages.remove(p)
@@ -468,6 +487,11 @@ class NattkaCommands(object):
         end_time = datetime.datetime.utcnow()
         log.info(f'Time elapsed: {end_time - start_time}')
         log.info(f'Target CC: {" ".join(cc_arches)}')
+
+        if self.args.stabilization:
+            log.warning('The package list contains newest versions visible.')
+            log.warning('Please adjust the package list to desired versions.')
+
         log.info('Package list follows:')
         print(f'{packages[0]} {initial_arches}')
         print('\n'.join(f'{x} ^' for x in packages[1:]))
@@ -545,7 +569,6 @@ class NattkaCommands(object):
             log.info(f'... will process until {end_time}')
 
         bz = self.get_bugzilla(require_api_key=self.args.update_bugs)
-        username = bz.whoami()
         bugnos, bugs = self.find_bugs()
         log.info(f'Found {len(bugnos)} bugs')
         bugs_done = 0
@@ -562,6 +585,13 @@ class NattkaCommands(object):
                     break
 
                 b = bugs[bno]
+                # Bugzilla is prone to race conditions between fetching bug
+                # data and updating bugs, so ignore bugs that have been updated
+                # recently.
+                if ((start_time - b.last_change_time).total_seconds() < 60
+                        and self.args.update_bugs):
+                    log.info(f'Bug {bno}: skipping due to recent change')
+                    continue
                 if b.category is None:
                     log.info(f'Bug {bno}: neither stablereq nor keywordreq')
                     continue
@@ -582,6 +612,7 @@ class NattkaCommands(object):
                 cc_arches: typing.List[str] = []
                 allarches_chg = False
                 expanded_plist: typing.Optional[str] = None
+                need_security_kw = False
 
                 try:
                     arches_cced = bool(
@@ -604,7 +635,7 @@ class NattkaCommands(object):
                                     f'package masked: {p.cpvstr}, '
                                     f'by keywords: {" ".join(mask_kws)}')
                             plist[p] = kw
-                    except KeywordNotSpecified:
+                    except (KeywordNotSpecified, KeywordNoneLeft):
                         assert not arches_cced
                         assert plist
                         # this is raised after iterating all entries,
@@ -632,7 +663,7 @@ class NattkaCommands(object):
                                 plist,
                                 match_package_list(
                                     repo, bugs[kw_dep], only_new=True))
-                        except KeywordNotSpecified:
+                        except (KeywordNotSpecified, KeywordNoneLeft):
                             raise DependentBugError(
                                 f'dependent bug #{kw_dep} is missing keywords')
                         except PackageListEmpty:
@@ -643,17 +674,36 @@ class NattkaCommands(object):
                                 f'dependent bug #{kw_dep} has errors')
 
                     # check if we have arches to CC
-                    if 'CC-ARCHES' in b.keywords and not arches_cced:
+                    if ('CC-ARCHES' in b.keywords and not arches_cced
+                            and b.assigned_to != 'bug-wranglers@gentoo.org'):
                         cc_arches = sorted(
                             [f'{x}@gentoo.org' for x
                              in set(filter_prefix_keywords(
                                  itertools.chain.from_iterable(
                                      check_packages.values())))])
+
                     # check if we have ALLARCHES to toggle
                     allarches = (b.category == BugCategory.STABLEREQ
-                                 and all(is_allarches(x) for x in plist))
+                                 and all(is_allarches(x) for x in plist)
+                                 and can_allarches_for_keywords(
+                                     repo, check_packages.items()))
                     allarches_chg = (allarches != ('ALLARCHES' in b.keywords))
 
+                    # check if we should add SECURITY keyword
+                    if not b.security and 'SECURITY' not in b.keywords:
+                        # SECURITY keyword doesn't apply to bugs in security
+                        # product
+                        for blocked_no in b.blocks:
+                            try:
+                                blocked_bug = bugs[blocked_no]
+                            except KeyError:
+                                blocked_bug = (
+                                    self.get_bugzilla()
+                                    .find_bugs(bugs=[blocked_no])[blocked_no])
+                            if blocked_bug.security:
+                                need_security_kw = True
+                                break
+
                     # check if keywords need expanding
                     if (('*' in b.atoms or '^' in b.atoms)
                             and (arches_cced or cc_arches)):
@@ -728,11 +778,12 @@ class NattkaCommands(object):
                     # do not update bug status, it's probably done
                     log.info('Skipping, no CC and probably no work to do')
                     continue
-                except KeywordNotSpecified:
+                except KeywordNotSpecified as e:
+                    e_packages = '\n'.join(f'- {x}' for x in e.pkgs)
                     log.info('Skipping because of incomplete keywords')
-                    comment = ('Resetting sanity check; keywords are '
-                               'not fully specified and arches are not '
-                               'CC-ed.')
+                    comment = (f'Keywords are not fully specified and '
+                               f'arches are not CC-ed for the following '
+                               f'packages:\n\n{e_packages}')
                     assert check_res is None
                 except PackageListDoneAlready:
                     # do not update bug status if done already
@@ -748,12 +799,14 @@ class NattkaCommands(object):
                     check_res = False
                     comment = f'Unable to check for sanity:\n\n> {e}'
                 except NoChanges:
-                    # if it's not positive, don't do extra work
-                    if b.sanity_check is not True:
-                        continue
-                    # check if there's anything related to do
-                    if not cc_arches and expanded_plist is None:
-                        continue
+                    # check if we need to add SECURITY keyword
+                    if not need_security_kw:
+                        # if it's not positive, don't do extra work
+                        if b.sanity_check is not True:
+                            continue
+                        # check if there's anything related to do
+                        if not cc_arches and expanded_plist is None:
+                            continue
                     check_res = True
                 except GitDirtyWorkTree:
                     log.critical(
@@ -775,7 +828,7 @@ class NattkaCommands(object):
                 # needs to change
                 if check_res is False and b.sanity_check is False:
                     assert comment is not None
-                    old_comment = bz.get_latest_comment(bno, username)
+                    old_comment = bz.get_latest_comment(bno)
                     # do not add a second identical comment
                     if (old_comment is not None
                             and comment.strip() == old_comment.strip()):
@@ -799,6 +852,8 @@ class NattkaCommands(object):
                 if allarches_chg:
                     log.info(f'{"Adding" if allarches else "Removing"} '
                              f'ALLARCHES')
+                if need_security_kw:
+                    log.info('Adding SECURITY keyword')
                 if expanded_plist:
                     log.info('Expanding package list')
                     if not self.args.update_bugs:
@@ -807,11 +862,16 @@ class NattkaCommands(object):
                     kwargs = {}
                     if cc_arches:
                         kwargs['cc_add'] = cc_arches
+                    keywords_add = []
                     if allarches_chg:
                         if allarches:
-                            kwargs['keywords_add'] = ['ALLARCHES']
+                            keywords_add.append('ALLARCHES')
                         else:
                             kwargs['keywords_remove'] = ['ALLARCHES']
+                    if need_security_kw:
+                        keywords_add.append('SECURITY')
+                    if keywords_add:
+                        kwargs['keywords_add'] = keywords_add
                     if expanded_plist:
                         kwargs['new_package_list'] = [expanded_plist]
                     bz.update_status(bno, check_res, comment,
@@ -915,6 +975,8 @@ def main(argv: typing.List[str]) -> int:
     mkpp.add_argument('-a', '--arch', action='append',
                       help='arch to keyword the first package for '
                            '(default: "*")')
+    mkpp.add_argument('-s', '--stabilization', action='store_true',
+                      help='prepare a package list for stabilization')
     mkpp.add_argument('package', nargs='+',
                       help='packages to keyword, first being the base '
                            'package (to which -a applies), '
