@@ -1,8 +1,9 @@
-# (c) 2020 Michał Górny
+# (c) 2020-2021 Michał Górny
 # 2-clause BSD license
 
 """ Tests for Bugzilla interaction. """
 
+import datetime
 import typing
 import unittest
 
@@ -40,7 +41,10 @@ class BugzillaTests(unittest.TestCase):
                  req: typing.Iterable[int]
                  ) -> typing.Dict[int, BugInfo]:
         """Return expected data for specified bugs"""
-        bugs = {1: BugInfo(None, '\r\n', blocks=[2]),
+        bugs = {1: BugInfo(None, '\r\n', blocks=[2],
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 3, 13, 22, 41)),
                 2: BugInfo(BugCategory.KEYWORDREQ,
                            'dev-python/unittest-mixins-1.6\r\n'
                            'dev-python/coverage-4.5.4\r\n',
@@ -48,37 +52,61 @@ class BugzillaTests(unittest.TestCase):
                                                         'hppa')],
                            depends=[1],
                            blocks=[9],
-                           sanity_check=True),
+                           sanity_check=True,
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 3, 13, 34, 59)),
                 3: BugInfo(BugCategory.STABLEREQ,
                            'dev-python/mako-1.1.0 amd64\r\n',
                            [f'{x}@gentoo.org' for x in ('amd64',)],
                            depends=[7],
                            keywords=['STABLEREQ'],
-                           sanity_check=False),
+                           sanity_check=False,
+                           assigned_to='bug-wranglers@gentoo.org',
+                           last_change_time=datetime.datetime(
+                               2020, 11, 26, 9, 42, 55)),
                 4: BugInfo(BugCategory.KEYWORDREQ,
                            'dev-python/urllib3-1.25.8\r\n'
                            'dev-python/trustme-0.6.0\r\n'
                            'dev-python/brotlipy-0.7.0\r\n',
                            [f'{x}@gentoo.org' for x in ('hppa',)],
-                           keywords=['KEYWORDREQ']),
+                           keywords=['KEYWORDREQ'],
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 3, 13, 34, 55)),
                 5: BugInfo(BugCategory.STABLEREQ,
                            'app-arch/arj-3.10.22-r7 amd64 hppa\r\n',
                            ['test@example.com'],
                            whiteboard='test whiteboard',
-                           security=True),
+                           security=True,
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 10, 9, 47, 22)),
                 6: BugInfo(BugCategory.STABLEREQ,
                            'sys-kernel/gentoo-sources-4.1.6\r\n',
-                           security=True),
+                           security=True,
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 3, 13, 31, 19)),
                 7: BugInfo(BugCategory.STABLEREQ,
                            'dev-python/pytest-5.4.1\r\n',
                            blocks=[3],
-                           keywords=['ALLARCHES']),
+                           keywords=['ALLARCHES'],
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 3, 13, 28, 17)),
                 8: BugInfo(BugCategory.STABLEREQ,
                            'dev-lang/python-3.7.7\r\n',
-                           resolved=True),
+                           resolved=True,
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 4, 7, 7, 56)),
                 9: BugInfo(BugCategory.KEYWORDREQ,
                            'dev-python/frobnicate-11\r\n',
-                           depends=[2]),
+                           depends=[2],
+                           assigned_to='test@example.com',
+                           last_change_time=datetime.datetime(
+                               2020, 4, 5, 14, 35, 59)),
                 }
         for k in list(bugs):
             if k not in req:
@@ -89,6 +117,7 @@ class BugzillaTests(unittest.TestCase):
     def test_whoami(self):
         """ Test whoami(). """
         self.assertEqual(self.bz.whoami(), BUGZILLA_USERNAME)
+        self.assertEqual(self.bz.username, BUGZILLA_USERNAME)
 
     @rec.use_cassette()
     def test_fetch_bugs(self):
@@ -122,14 +151,6 @@ class BugzillaTests(unittest.TestCase):
                                         BugCategory.STABLEREQ]),
             self.get_bugs([2, 3, 4, 8]))
 
-    @rec.use_cassette()
-    def test_fetch_bugs_security(self):
-        """Test getting and filtering to security bugs."""
-        self.assertEqual(
-            self.bz.find_bugs([3, 4, 6, 8],
-                              security=True),
-            self.get_bugs([6]))
-
     @rec.use_cassette()
     def test_fetch_sanity_check_passed(self):
         """Test filtering bugs by sanity-check+."""
@@ -176,54 +197,6 @@ class BugzillaTests(unittest.TestCase):
             self.bz.find_bugs(category=[BugCategory.STABLEREQ]),
             self.get_bugs([3, 5, 6, 7, 8]))
 
-    @rec.use_cassette()
-    def test_find_security(self):
-        """ Test finding security bugs. """
-        self.assertEqual(
-            self.bz.find_bugs(security=True),
-            self.get_bugs([5, 6]))
-
-    @rec.use_cassette()
-    def test_find_security_keywordreq(self):
-        """ Test finding security keywordreq bugs (no such thing). """
-        self.assertEqual(
-            self.bz.find_bugs(category=[BugCategory.KEYWORDREQ],
-                              security=True),
-            {})
-
-    @rec.use_cassette()
-    def test_find_security_stablereq(self):
-        """ Test finding security stablereq bugs (all of them). """
-        self.assertEqual(
-            self.bz.find_bugs(category=[BugCategory.STABLEREQ],
-                              security=True),
-            self.get_bugs([5, 6]))
-
-    @rec.use_cassette()
-    def test_find_security_both(self):
-        """ Test finding security keywordreq and stablereq bugs. """
-        self.assertEqual(
-            self.bz.find_bugs(category=[BugCategory.KEYWORDREQ,
-                                        BugCategory.STABLEREQ],
-                              security=True),
-            self.get_bugs([5, 6]))
-
-    @rec.use_cassette()
-    def test_find_nonsecurity_keywordreqs(self):
-        """ Test finding keywordreqs that are not security bugs. """
-        self.assertEqual(
-            self.bz.find_bugs(category=[BugCategory.KEYWORDREQ],
-                              security=False),
-            self.get_bugs([2, 4, 9]))
-
-    @rec.use_cassette()
-    def test_find_nonsecurity_stablereqs(self):
-        """ Test finding non-security stablereqs. """
-        self.assertEqual(
-            self.bz.find_bugs(category=[BugCategory.STABLEREQ],
-                              security=False),
-            self.get_bugs([3, 7, 8]))
-
     @rec.use_cassette()
     def test_find_bugs_cc(self):
         """Test finding bugs by CC."""
@@ -281,6 +254,13 @@ class BugzillaTests(unittest.TestCase):
             self.bz.get_latest_comment(3, BUGZILLA_USERNAME),
             'sanity check failed!')
 
+    @rec.use_cassette()
+    def test_get_latest_comment_whoami(self):
+        """ Test getting latest self-comment with whoami(). """
+        self.assertEqual(
+            self.bz.get_latest_comment(3),
+            'sanity check failed!')
+
 
 class DestructiveBugzillaTests(unittest.TestCase):
     bz: NattkaBugzilla
@@ -306,6 +286,23 @@ class DestructiveBugzillaTests(unittest.TestCase):
         self.assertIsNone(
             self.bz.get_latest_comment(5, BUGZILLA_USERNAME))
 
+    @rec.use_cassette()
+    def test_set_status_and_mark_obsolete(self):
+        """Test setting sanity-check status and marking comments obsolete"""
+        self.assertFalse(
+            self.bz.find_bugs([3])[3].sanity_check,
+            'Bugzilla instance tainted, please reset')
+        self.assertIsNotNone(
+            self.bz.get_latest_comment(3, BUGZILLA_USERNAME),
+            'Bugzilla instance tainted, please reset')
+
+        self.bz.update_status(3, False)
+
+        self.assertFalse(
+            self.bz.find_bugs([3])[3].sanity_check)
+        self.assertIsNotNone(
+            self.bz.get_latest_comment(3, BUGZILLA_USERNAME))
+
     @rec.use_cassette()
     def test_set_status_and_comment(self):
         """Test setting sanity-check status and commenting"""
